"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const execa_1 = __importDefault(require("execa"));
const fs_1 = __importDefault(require("fs"));
const ora_1 = __importDefault(require("ora"));
const path_1 = __importDefault(require("path"));
const util_1 = require("util");
const RearguardConfig_1 = require("../../../configs/RearguardConfig");
const const_1 = require("../../../const");
const bundleNaming_1 = require("../../../helpers/bundleNaming");
const dependencyPaths_1 = require("../../../helpers/dependencyPaths");
const getSortedListOfDependencies_1 = require("../getSortedListOfDependencies");
const exist = util_1.promisify(fs_1.default.exists);
exports.buildUnfinishedDependencies = async (CWD) => {
    const dependencies = await getSortedListOfDependencies_1.getSortedListOfDependencies(CWD);
    const globalNodeModulePath = dependencyPaths_1.getGlobalNodeModulePath();
    const localNodeModulePath = dependencyPaths_1.getLocalNodeModulePath(CWD);
    const unfinishedDependencies = new Set();
    for (const dependency of dependencies) {
        let dependencyPath = "";
        const dependencyGlobalPath = path_1.default.resolve(globalNodeModulePath, dependency);
        const dependencyLocalPath = path_1.default.resolve(localNodeModulePath, dependency);
        if (await exist(dependencyLocalPath)) {
            dependencyPath = dependencyLocalPath;
        }
        else if (await exist(dependencyGlobalPath)) {
            dependencyPath = dependencyGlobalPath;
        }
        if (dependencyPath === "") {
            console.log(chalk_1.default.red.bold(`!!! ERROR !!!`));
            console.log("");
            console.log(chalk_1.default.red(`${dependency} not found`));
            console.log("");
            console.log(chalk_1.default.red(`Global path to dependency: ${dependencyGlobalPath}`));
            console.log(chalk_1.default.red(`Local path to dependency: ${dependencyLocalPath}`));
            console.log("");
            console.trace(chalk_1.default.red.bold(`Path to ${dependency} not found`));
            process.exit(1);
        }
        if (!(await exist(dependencyGlobalPath))) {
            continue;
        }
        const rearguardConfig = new RearguardConfig_1.RearguardConfig(dependencyGlobalPath);
        const pkgFiles = rearguardConfig.getFiles();
        const snakeName = rearguardConfig.getSnakeName();
        for (const pkgFile of pkgFiles) {
            const pkgFilePath = path_1.default.resolve(dependencyGlobalPath, pkgFile);
            if (pkgFile === const_1.DLL_BUNDLE_DIR_NAME || pkgFile === const_1.LIB_BUNDLE_DIR_NAME) {
                if (!(await exist(pkgFilePath))) {
                    unfinishedDependencies.add(dependencyGlobalPath);
                }
                else {
                    const devSubDir = path_1.default.resolve(dependencyGlobalPath, pkgFile, snakeName, bundleNaming_1.getBundleSubDir(true));
                    const prodSubDir = path_1.default.resolve(dependencyGlobalPath, pkgFile, snakeName, bundleNaming_1.getBundleSubDir(false));
                    if (!(await exist(devSubDir)) || !(await exist(prodSubDir))) {
                        unfinishedDependencies.add(dependencyGlobalPath);
                    }
                    else if ((await exist(devSubDir)) && fs_1.default.readdirSync(devSubDir).length === 0) {
                        unfinishedDependencies.add(dependencyGlobalPath);
                    }
                    else if ((await exist(prodSubDir)) && fs_1.default.readdirSync(prodSubDir).length === 0) {
                        unfinishedDependencies.add(dependencyGlobalPath);
                    }
                }
            }
            else if (pkgFile === const_1.LIB_DIR_NAME) {
                if (!(await exist(pkgFilePath))) {
                    unfinishedDependencies.add(dependencyGlobalPath);
                }
                else if (fs_1.default.readdirSync(pkgFilePath).length === 0) {
                    unfinishedDependencies.add(dependencyGlobalPath);
                }
            }
        }
    }
    if (unfinishedDependencies.size > 0) {
        console.log(chalk_1.default.bold.blue(`[ BUILD UNFINISHED DEPENDENCIES ]`));
        console.log("");
        for (const dependency of unfinishedDependencies) {
            const dirNames = dependency.split("/");
            console.log(chalk_1.default.blue(`${dirNames[dirNames.length - 1]}: ${dependency}`));
        }
        console.log("");
        for (const dependency of unfinishedDependencies) {
            const dirNames = dependency.split("/");
            const spinner = ora_1.default(`build: ${dirNames[dirNames.length - 1]}`).start();
            try {
                await execa_1.default("npm", ["run", "build", "--", "--bypass_the_queue"], { cwd: dependency });
                if (spinner) {
                    spinner.succeed();
                }
            }
            catch (error) {
                if (spinner) {
                    spinner.fail();
                }
                console.error(error);
            }
        }
    }
};
//# sourceMappingURL=buildUnfinishedDependencies.js.map