"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const prettier_1 = __importDefault(require("prettier"));
const prettier_package_json_1 = __importDefault(require("prettier-package-json"));
const semver_1 = __importDefault(require("semver"));
const lodash_1 = require("lodash");
const const_1 = require("../const");
const PackageJSON_1 = require("./PackageJSON");
class PackageJSONConfig {
    constructor(CWD = process.cwd()) {
        this.CWD = CWD;
        this.packageJsonFileName = "package.json";
        this.pathToPackageJsonFile = path_1.default.resolve(this.CWD, this.packageJsonFileName);
    }
    getPkg() {
        try {
            if (fs_1.default.existsSync(this.pathToPackageJsonFile)) {
                const origin = JSON.parse(fs_1.default.readFileSync(this.pathToPackageJsonFile, { encoding: "utf-8" }));
                return new PackageJSON_1.PackageJSON(origin);
            }
            else {
                console.trace(`File ${this.pathToPackageJsonFile} not found`);
                process.exit(1);
            }
        }
        catch (error) {
            console.error(error);
            process.exit(1);
        }
        return new PackageJSON_1.PackageJSON({});
    }
    getName() {
        return this.getPkg().name;
    }
    getSnakeName() {
        return lodash_1.snakeCase(this.getName());
    }
    getVersion() {
        return this.getPkg().version;
    }
    async setVersion(version) {
        const origin = this.getPkg();
        const pkg = new PackageJSON_1.PackageJSON({ ...origin, version });
        return this.setPkg(pkg);
    }
    getFiles() {
        return this.getPkg().files;
    }
    getBrowserslist() {
        return this.getPkg().browserslist;
    }
    async setFiles(files) {
        const origin = this.getPkg();
        const pkg = new PackageJSON_1.PackageJSON({ ...origin, files });
        return await this.setPkg(pkg);
    }
    async setTypes(types) {
        const origin = this.getPkg();
        const pkg = new PackageJSON_1.PackageJSON({ ...origin, types });
        return await this.setPkg(pkg);
    }
    async setMain(main) {
        const origin = this.getPkg();
        const pkg = new PackageJSON_1.PackageJSON({ ...origin, main });
        return await this.setPkg(pkg);
    }
    getDependencies() {
        return this.getPkg().dependencies || {};
    }
    getDependencyList() {
        return Object.keys(this.getDependencies());
    }
    async setDependencies(dependencies) {
        return await this.setPkg(new PackageJSON_1.PackageJSON({ ...this.getPkg(), dependencies }));
    }
    async setDependencyVersion(dependencyName, version) {
        const pkg = this.getPkg();
        const dependencies = { ...pkg.dependencies };
        const devDependencies = { ...pkg.devDependencies };
        const peerDependencies = { ...pkg.peerDependencies };
        if (dependencies[dependencyName] &&
            dependencies[dependencyName].indexOf("file") === -1 &&
            semver_1.default.valid(version)) {
            dependencies[dependencyName] = version;
        }
        if (devDependencies[dependencyName] &&
            devDependencies[dependencyName].indexOf("file") === -1 &&
            semver_1.default.valid(version)) {
            devDependencies[dependencyName] = version;
        }
        if (peerDependencies[dependencyName] &&
            peerDependencies[dependencyName].indexOf("file") === -1 &&
            semver_1.default.valid(version)) {
            peerDependencies[dependencyName] = version;
        }
        return await this.setPkg(new PackageJSON_1.PackageJSON({ ...this.getPkg(), dependencies, devDependencies, peerDependencies }));
    }
    getDevDependencies() {
        return this.getPkg().devDependencies || {};
    }
    getDevDependencyList() {
        return Object.keys(this.getDevDependencies());
    }
    async setDevDependencies(devDependencies) {
        return await this.setPkg(new PackageJSON_1.PackageJSON({ ...this.getPkg(), devDependencies }));
    }
    getPeerDependencies() {
        return this.getPkg().peerDependencies || {};
    }
    getPeerDependencyList() {
        return Object.keys(this.getPeerDependencies());
    }
    async setPeerDependencies(peerDependencies) {
        return await this.setPkg(new PackageJSON_1.PackageJSON({ ...this.getPkg(), peerDependencies }));
    }
    getScripts() {
        return this.getPkg().scripts;
    }
    async setScripts(scripts, force = false) {
        const origin = this.getPkg();
        const pkg = new PackageJSON_1.PackageJSON({
            ...origin,
            scripts: force ? scripts : { ...origin.scripts, ...scripts },
        });
        return await this.setPkg(pkg);
    }
    getRearguard() {
        return this.getPkg().rearguard;
    }
    async setRearguard(rearguard) {
        return await this.setPkg(new PackageJSON_1.PackageJSON({ ...this.getPkg(), rearguard }));
    }
    async reformat() {
        await this.setPkg(this.getPkg());
    }
    async setPkg(origin) {
        try {
            if (fs_1.default.existsSync(this.pathToPackageJsonFile)) {
                const formatedJSON = JSON.parse(prettier_package_json_1.default.format(origin.toJSON()));
                formatedJSON.scripts = origin.scripts;
                const content = prettier_1.default.format(JSON.stringify(formatedJSON), const_1.PRETTIER_JSON_STRINGIFY);
                fs_1.default.writeFileSync(this.pathToPackageJsonFile, content, { encoding: "utf-8" });
            }
            else {
                console.trace(`File ${this.pathToPackageJsonFile} not found`);
                process.exit(1);
            }
        }
        catch (error) {
            console.error(error);
            process.exit(1);
        }
        return origin;
    }
}
exports.PackageJSONConfig = PackageJSONConfig;
//# sourceMappingURL=PackageJSONConfig.js.map